
jQuery(document).ready(function($){
    google.charts.load('current', {'packages':['gauge']});

    var container = $("#kurtztest_survey");
    var baseUrl = container.data('base-url');
    var accessCode = container.data('access-code');
    var apiToken = "API-X-TOKEN " + container.data('mind-one-api-key');

    var initButton = $("<button></button>").addClass('btn btn-default').html("Start");
    initButton.on('click', init);

    var surveyID, surveyData, surveyReportData;

    container.css({
        'font-family': "'Montserrat', sans-serif"
    });

    if(sessionStorage.getItem('surveyID'))
    {
        surveyID = sessionStorage.getItem('surveyID');
        getQuestionnaire();
    }else{
        initButton.appendTo(container);
    }

    function init() {
        initButton.attr('disabled', true);
        $.ajax({
            url: baseUrl + 'api/init_test',
            type: "POST",
            data: {
                accessCode: accessCode
            },
            beforeSend: function(jqxhr) {
                jqxhr.setRequestHeader("X-AUTH-TOKEN", apiToken);
            },
            success: function(data){
                surveyID = JSON.parse(data).SurveyId;
                sessionStorage.setItem('surveyID', surveyID);
                getQuestionnaire();
            },
            error: function(jqxhr, textStatus, errorThrown) {},
            complete: function(jqxhr, textStatus) {}
        });
    }

    function getQuestionnaire(){
        $.ajax({
            url: baseUrl + 'api/get_questionnaire',
            type: "GET",
            data: {
                SurveyId: surveyID
            },
            beforeSend: function(jqxhr) {
                jqxhr.setRequestHeader("X-AUTH-TOKEN", apiToken);
            },
            success: function(data){
                surveyData = JSON.parse(data);
                surveyData.Status === "Completed" ?
                renderResults(surveyData) : renderSurvey(surveyData);
                makeAllCellsSameHeigthInRow();
            },
            error: function(jqxhr, textStatus, errorThrown) {

            },
            complete: function(jqxhr, textStatus) {
    
            }
        });
    }

    function renderSurvey(data) {
        container.html("");
        var surveyForm = $("<form></form>")
            .attr('name', "survey")
            .attr('id', "survey")
            .attr('action', baseUrl)
            .attr('method', "POST")
            .css("margin-bottom", "10px");
        container.append(surveyForm);
        var surveyHead = $("<div></div>").addClass('survey_head').appendTo(surveyForm);
        var infobox = $("<div></div>").addClass("infobox").appendTo(surveyHead);
        var infoboxRow = $("<div></div>").addClass("row").appendTo(infobox);
        var infoboxCol = $("<div></div>").addClass("col-sm-12").appendTo(infoboxRow);
        var infoboxQuestion = $("<div></div>").addClass("question").html(data.IntroductionText);
        infoboxCol.append(infoboxQuestion);
        var scaleWrapper = $("<div></div>").addClass('scale-wrapper').appendTo(surveyForm);

        $.each(data.items, function(key, item){
            let rowScale = $("<div></div>").addClass('row-scale form_field').appendTo(scaleWrapper);
            let questionWrapper = $("<div></div>")
                .addClass("col-sm-12 col-md-6 col-lg-8 nopadding").appendTo(rowScale);
            let labelCol = $("<div></div>")
                .addClass("col-xs-1 border-b follow-label-height-numbering")
                .appendTo(questionWrapper);
            let jsNumbering = $("<div></div>").addClass("js-numbering").html(key + 1)
                .appendTo(labelCol);
            let scaleLabel = $("<div></div>").addClass("col-xs-11 border-rbl scale-label")
                .appendTo(questionWrapper).html(item.label);
            let errorMsgHolder = $("<div></div>").addClass("error_message_holder")
                .appendTo(scaleLabel);
            let answersWrapper = $("<div></div>").addClass('col-sm-12 col-md-6 col-lg-4 nopadding')
                .appendTo(rowScale);
            let tableDiv = $("<div></div>").addClass('table-div').css("width", "100%")
                .appendTo(answersWrapper);
            let trDiv = $("<div></div>").addClass("tr-div").appendTo(tableDiv);

            $.each(item.answerLabels[0], function(index, ansLabel){
                let tdDivRadio = $("<div></div>").addClass("td-div radio").appendTo(trDiv);
                let radioInput = $("<input/>")
                    .attr("type", "radio").addClass("required")
                    .attr('name', 'level_' + item.id)
                    .attr("id", "level_" + item.id + "_" + index)
                    .attr('value', ansLabel.answerValue).appendTo(tdDivRadio);
                let radioLabel = $("<label></label>").addClass('follow-label-height')
                    .attr('for', "level_" + item.id + "_" + index)
                    .attr('id', "label_level_" + item.id + "_" + index)
                    .attr('title', ansLabel.answerLabel).html(ansLabel.answerValue)
                    .appendTo(tdDivRadio);

                if(item.answer && parseInt(item.answer) === parseInt(ansLabel.answerValue)){
                    radioInput.prop('checked', true);
                }
                
                radioInput.on('click', function(){
                    var siblingRadios = $("input[name="+radioInput.attr('name')+"]");
                    siblingRadios.attr("disabled", true);
                    $.ajax({
                        url: baseUrl + 'api/save_answer',
                        type: "POST",
                        beforeSend: function(jqxhr) {
                            jqxhr.setRequestHeader("X-AUTH-TOKEN", apiToken);
                        },
                        data: {
                            itemId: item.id,
                            answer: ansLabel.answerValue,
                            SurveyId: data.SurveyId
                        },
                        success: function(data){
                            console.log(data);
                            siblingRadios.attr("disabled", false);
                        },
                        error: function(jqxhr, textStatus, errorThrown) {
                            alert( "Error saving answer. The page will reload." );
                            location.reload();
                        },
                    });
                });
            });
        });

        var submitButton = $("<button></button>").addClass('btn btn-primary')
            .css('margin-top', '20px').attr('id', 'submit').attr('name', 'submit')
            .attr('type', 'button')
            .html("Submit").appendTo(surveyForm);

        var submitErrorMsg = $("<div></div>").addClass('submit-error-msg').appendTo(surveyForm);

        submitButton.on('click', function(){
            $(".submit-error-msg").html("");
            if(!validateAnswers()) {
                $(".submit-error-msg").html("Please provide an answer for all of the items above.");
                return false;
            }
            submitButton.attr('disabled', true);
            renderResults(data);
        });
    }

    function renderResults(data) {
        container.html("");
        $.ajax({
            url: baseUrl + 'api/display_results',
            type: "POST",
            data: {
                SurveyId: data.SurveyId
            },
            beforeSend: function(jqxhr) {
                jqxhr.setRequestHeader("X-AUTH-TOKEN", apiToken);
            },
            success: function(data){
                console.log(data);
                surveyReportData = data;
                google.charts.setOnLoadCallback(drawChart);
                $("#submit").attr('disabled', true);
            },
            error: function(jqxhr, textStatus, errorThrown) {
                $("#submit").attr('disabled', true);
            },
        });
    }

    function drawChart() {
        console.log(surveyReportData.levels[0].score);
        var chartContainer = document.createElement('div');
        $(chartContainer).css({
            'width' : 500,
            'margin': '5px auto'
        });

        var mainCompetenceHeading = $("<h3></h3>").addClass('text-center')
            .html(surveyReportData.levels[0].label);

        var mainCompetenceParagraph = $("<p></p>")
            .html(surveyReportData.levels[0].scoreText)
            .css({
                'width' : 900,
                'margin': '5px auto',
                // 'font-size': '1.3em'
            });;

        var competenceScoresContainer = createCompetenceScoresContainer();

        container.html("")
            .append(mainCompetenceHeading)
            .append(mainCompetenceParagraph)
            .append($(chartContainer))
            .append(competenceScoresContainer);

        // surveyReportData.levels[0].label;

        var data = google.visualization.arrayToDataTable([
            ['Label', 'Value'],
            ['', surveyReportData.levels[0].score],
        ]);

        var options = {
            width: 400  , height: 400,
            redFrom:0, redTo: 65,
            yellowFrom:65, yellowTo: 85, 
            greenFrom: 85, greenTo: 100,           
            minorTicks: 5
        };

        var chart = new google.visualization.Gauge(chartContainer);
        chart.draw(data, options);
    }

    function createCompetenceScoresContainer() {
        var  competenceScoresContainer = $("<div></div>").css({
            'width' : 900,
            'margin': '5px auto',
        });
        

        $.each(surveyReportData.levels, function(key, level){
            if(key === 0) {return;}
            let competenceWrapper = $("<div></div>").css({
                'width': 290,
                'float': 'left'
            });
            let competenceLabel = $("<h4></h4>").html(level.label);
            let competenceScore = $("<span></span>").html(level.score)
                .addClass(getLabelClass(level.score)).css('margin-left', '5px');
            let competenceScoreText = $("<p></p>").html(level.scoreText)
                .css('margin', '20px 10px 20px 0px');
            competenceLabel.append(competenceScore);
            competenceWrapper.append(competenceLabel).append(competenceScoreText);
            competenceScoresContainer.append(competenceWrapper);
        });

        return competenceScoresContainer;
    }

    function getLabelClass(score) {
        switch (true) {
            case score < 65:
                return "label label-danger";
            case score < 85:
                return "label label-warning";            
            case score <= 100:
                return "label label-success";        
            default:
                return "";
        }
    }

    function validateAnswers() {
        var unansweredItems = 0;
        $(".row-scale.form_field").each(function(index, element){
            if($(element).find(".required:checked").length > 0)
            {
                $(element).removeClass('text-danger')
            }else{
                $(element).addClass('text-danger');
                unansweredItems++;
            }
        });

        return unansweredItems === 0;
    }

    function makeAllCellsSameHeigthInRow(){
        $('.follow-label-height').each(function(){
            var divLabel = $(this).closest('.row-scale').find('.scale-label');
            divLabel.css('height', '');
            var divHeight = Math.round(divLabel.height())+16;
            divLabel.css('height', (divHeight+1)+'px');
            $(this).css('height', divHeight+'px');
        });

        $('.follow-label-height-numbering').each(function(){
            var divHeight = $(this).closest('.row-scale').find('.scale-label').height() +17;
            $(this).css('height', divHeight+'px');
        });
    }

    $(window).resize(function(){
        makeAllCellsSameHeigthInRow();  
    });

    makeAllCellsSameHeigthInRow();

});
